/*
 * source code positions
 *
 * 20.03.2001, Christoph Zenger, Matthias Zenger
 */

package expression;

import java.io.*;

/** class for sourcecode positions
 *  columns are only correct up to 0xfff
 */
public class Position {
    
    /** undefined position
     */
    public static final int UNDEFINED = 1;
    
    /** first position in a source file
     */
    public static final int FIRST = (1 << 12) | 1;
    
    /** encode a line and column number into a single int
     */
    public static int encode(int line, int col) {
	return (line << 12) | (col & 0xfff);
    }
    
    /** get the line number out of an encoded position
     */
    public static int line(int pos) {
	return pos >>> 12;
    }
    
    /** return the column number of an encoded position
     */
    public static int column(int pos) {
	return pos & 0xfff;
    }
    
    /** give a string representation of the encoded position
     */
    public static String toString(int pos) {
	if (pos == UNDEFINED) 
	    return "?:?";
	else
	    return line(pos) + ":" + column(pos);
    }
}












