/**
 * A scanner for expressions
 */

package expression;
import java.io.*;


/** the scanner class
 */
class Scanner implements Tokens {
     
    /** the token as read from the input
     */
    public String chars;
        
    /** the current lookahead character
     */
    protected int ch;

    /** line and column of ch
     */
    protected int line, col;

    /** position of current token
     */
    public int pos;
        
    /** this method scans the next token
     */
    public int nextToken() throws IOException {
	while (ch == ' ' || ch == '\t'
	       || ch == '\n' || ch == '\r') {
	    ch = nextCh();
	}
	pos = Position.encode(line, col);
	switch(ch) {
	case -1:
	    return EOF; 
	case '0': case '1': case '2': case '3': case '4': 
	case '5': case '6': case '7': case '8': case '9':
	    chars = "" + (char) ch;
	    ch = nextCh();
	    while (ch >= '0' && ch <= '9') {
		chars += (char) ch;
		ch = nextCh();
	    }
	    return INTLIT;
	case '(': 
	    ch = nextCh();
	    return LPAREN;
	case ')': 
	    ch = nextCh();
	    return RPAREN;
	case '+': 
	    ch = nextCh();
	    return PLUS;
	case '-': 
	    ch = nextCh();
	    return MINUS;
	case '*': 
	    ch = nextCh();
	    return TIMES;
	case '/': 
	    ch = nextCh();
	    return DIV;
	case '=': 
	    ch = nextCh();
	    return EQUALS;
	case ';': 
	    ch = nextCh();
	    return SEMI;
	default:
	    if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')) {
		chars = "" + (char) ch;
		ch = nextCh();
		while ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')) {
		    chars = "" + (char) ch;
		    ch = nextCh();
		}
		return IDENT;
	    } else {
		ch = nextCh();
		return ERROR;
	    }
	}
    }

    /** the input stream
     */
    protected InputStream in;

    /** read next character and return it
     */
    protected int nextCh() throws IOException {
	int ch = in.read();
	if (ch == '\n') {
	    line ++;
	    col = 0;
	} else {
	    col ++;
	}
	return ch;
    }

    /** a new scanner that reads from in
     */
    public Scanner(InputStream in) throws IOException {
        this.in = in;
	line = 1;
	col = 0;
	// read first character
        ch = nextCh();
    }

    /** print the string corresponding to token 
     *  (for debugging)
     */
    public static String toString(int token, String s) {
	switch(token) {
	case INTLIT:
	    return "<int> " + s;
	case IDENT:
	    return "<ident> " + s;
	case LPAREN:
	    return "(";
	case RPAREN:
	    return ")";
	case PLUS:
	    return "+";
	case MINUS:
	    return "-";
	case DIV:
	    return "/";
	case TIMES:
	    return "*";
	case EQUALS:
	    return "=";
	case SEMI:
	    return ";";
	case EOF:
	    return "<eof>";
	case ERROR:
	    return "<error>";
	default:
	    return "<unknown>";
	}
    }

    /** main function for testing scanner
     *  (for debugging)
     */
    static public void main(String[] args) {
        try {
            Scanner s = new Scanner(System.in);
	    int tok;
	    do {
		tok = s.nextToken();
                System.out.println(Position.toString(s.pos) + ": " + s.toString(tok, s.chars));
            } while (tok != EOF);
        } catch (IOException e) {
            System.out.println(e.toString());
        }
    }
}           




